*** Settings ***
Library    Browser
Library    Dialogs
Library    String
Library    Collections
Library    DateTime
Library    OperatingSystem
Library    Process
Resource    Variables.robot

Suite Setup   Run Keywords     Set Executable Status
...    AND    Suite Can Continue
...    AND    New Browser    downloadsPath=${OUTPUTDIR}     headless=no
...    AND    New Context    acceptDownloads=yes
...    AND    New Page    ${HOME_PAGE}
...    AND    Go To Login Page
...    AND    Set Date Range
...    AND    Enter Username And Password
...    AND    Login
Suite Teardown  Run Keywords    Pass Execution If Skipped
...    AND    Go To    ${LANDING_PAGE}
...    AND    Capture Account Balances
...    AND    Close Browser
Task Setup    Go To Search And Export Page
Task Template    Download Statement


*** Variables ***
${HOME_PAGE}    https://www.asb.co.nz
${EXPORT_PREFIX}    Export
${SAVE_PREFIX}    Robot
${SAVE_TYPE}    ofx
${SAVE_SUBTYPE}    MS Money
${LANDING_PAGE}    ${EMPTY}
${FROM_DATE}    ${EMPTY}
${TO_DATE}    ${EMPTY}
${SKIP}    no

*** Keywords ***
New Page
    [Arguments]    ${url}    @{args}    &{kwargs}
    ${old_timeout} =    Set Browser Timeout    30 seconds
    Browser.New Page    ${url}    @{args}    &{kwargs}
    [Teardown]    Set Browser Timeout    ${old_timeout}

Page Should Contain
    [Arguments]    ${expected}
    ${page}=    Get Page Source
    Should Contain    ${page}    ${expected}
    
Output Directory Is Dirty
    File Should Exist    ${OUTPUTDIR}/${EXPORT_PREFIX}*.${SAVE_TYPE}

Set Executable Status
    ${status}=    Run Keyword And Return Status    Output Directory Is Dirty
    ${SKIP}=    Set Variable If    ${status}   yes    no
    Set Suite Variable    ${SKIP}

Suite Can Continue
    ${SKIP}=    Evaluate    robot.utils.is_truthy('${SKIP}')    modules=robot.utils
    Run Keyword If    ${SKIP}    Fail    msg=Output directory is dirty
    
Pass Execution If Skipped
    ${SKIP}=    Evaluate    robot.utils.is_truthy('${SKIP}')    modules=robot.utils
    Pass Execution If    ${SKIP}   Suite is skipped.

Set Date Range
    Return From Keyword If    '${FROM_DATE}' != '${EMPTY}'
    ${TO_DATE}=    Get Current Date
    ${FROM_DATE}=    Subtract Time From Date     ${TO_DATE}    28 days    result_format=%d/%m/%Y
    ${TO_DATE}=    Convert Date    ${TO_DATE}    result_format=%d/%m/%Y
    ${val}=    Get Value From User    Enter from date range    default_value=${FROM_DATE} ${TO_DATE}
    ${FROM_DATE}    ${TO_DATE}=    Split String    ${val}
    ${FROM_DATE}=    Convert Date    ${FROM_DATE}    date_format=%d/%m/%Y
    ${TO_DATE}=    COnvert Date    ${TO_DATE}    date_format=%d/%m/%Y
    Set Suite Variable    ${FROM_DATE}
    Set Suite Variable    ${TO_DATE}

Go To Login Page
    Click    xpath=//a[@class="btn login-button" and contains(text(), 'Log In')]
    
Go To Search And Export Page
    [Documentation]    Assumes freshly logged in.
    Click    xpath=//iframe[@id="common-header"] >>> xpath=//button[@id="main-menu-button"]
    Click    xpath=//iframe[@title="mainmenu"] >>> xpath=//button[contains(text(),'Everyday banking')]
    Click    xpath=//iframe[@title="mainmenu"] >>> xpath=//a[contains(.,'Search and export transactions')]
    Wait Until Keyword Succeeds    5x    1 second
    ...    Page Should Contain    Export Format
    Wait Until Keyword Succeeds    5x    1 second
    ...    Page Should Contain    Search and Export

Enter Username And Password
    [Documentation]    Enters user/password into text fields of login page.
    ${url}=    Get Url
    ${log_level}=    Set Log Level    NONE
    ${password}=    Get Value From User    Enter password for ${url}    hidden=yes
    Set Log Level    ${log_level}
    Fill Text    xpath=//input[@id="dUsername"]    ${USERNAME}
    Fill Secret    xpath=//input[@id="password"]    $password
    [Teardown]    Set Log Level    ${log_level}

Login
    [Documentation]    Login and Return Home URL. Assumes user/password already entered.
    Click    xpath=//button[@id="loginBtn"]
    Execute Manual Step    Complete 2-factor authentication
    Wait Until Keyword Succeeds    5x    1 second
    ...    Page Should Contain    ${ACCOUNT_HOLDER_NAME}
    ${url}=    Get Url
    ${parts}=    Split String    ${url}    /
    ${parts}=    Get Slice From List    ${parts}     0    -2
    ${LANDING_PAGE}=    Catenate    SEPARATOR=/    @{parts}
    Set Suite Variable    ${LANDING_PAGE}

Download Statement
    [Arguments]    ${account}
    Click    xpath=//*[@id="Request_AccountKey_input"]
    Click    xpath=//li[contains(.,'${account}')]
    ${FROM_DATE}=    Convert Date    ${FROM_DATE}    result_format=datetime
    ${TO_DATE}=    Convert Date    ${TO_DATE}    result_format=datetime
    ${from_date_year}=    Convert To String    ${FROM_DATE.year}
    ${from_date_month}=    Convert To String    ${FROM_DATE.month}
    ${from_date_day}=    Convert To String    ${FROM_DATE.day}
    ${to_date_year}=    Convert To String    ${TO_DATE.year}
    ${to_date_month}=    Convert To String    ${TO_DATE.month}
    ${to_date_day}=    Convert To String    ${TO_DATE.day}
    Fill Text    xpath=//*[@id="Request_FromDate_inputYear"]    ${FROM_DATE_year}
    Fill Text    xpath=//*[@id="Request_FromDate_inputMonth"]    ${FROM_DATE_month}
    Fill Text    xpath=//*[@id="Request_FromDate_inputDate"]    ${FROM_DATE_day}
    Fill Text    xpath=//*[@id="Request_ToDate_inputYear"]    ${TO_DATE_year}
    Fill Text    xpath=//*[@id="Request_ToDate_inputMonth"]    ${TO_DATE_month}
    Fill Text    xpath=//*[@id="Request_ToDate_inputDate"]    ${TO_DATE_day}
    Click    xpath=//*[@id="Request_ExportFormat_input"]
    Click    xpath=//li[contains(.,'${SAVE_TYPE.upper()}') and contains(.,'${SAVE_SUBTYPE}')]
    Promise To Wait For Download    ${OUTPUTDIR}/${EXPORT_PREFIX}-${account}.${SAVE_TYPE}
    Click    xpath=//input[@value="Export"]
    Wait For All Promises
    ${status}=    Run Keyword And Return Status
    ...    Wait Until Created    ${OUTPUTDIR}/${EXPORT_PREFIX}*.${SAVE_TYPE}    timeout=5 seconds
    Run Keyword If    not ${status}    Take Screenshot
    Run Keyword If    not ${status}    Fail    msg=No file downloaded for account: ${account}
    ${files}=    List Files In Directory    ${OUTPUTDIR}
    FOR    ${f}    IN    @{files}
        Continue For Loop If    not '${f}'.startswith('${EXPORT_PREFIX}')
        Move File    ${OUTPUTDIR}${/}${f}    ${OUTPUTDIR}${/}${SAVE_PREFIX}-${account}.${SAVE_TYPE}
        Return From Keyword
    END
    Fail    msg=Failed to save downloaded account statement to ${OUTPUTDIR}: ${account}

Capture Account Balances
    [Documentation]    Assumes current page is Balances.
    FOR    ${account}    IN    @{ACCOUNTS}
        Wait Until Keyword Succeeds    2x    1 second
        ...    Page Should Contain    ${account}
    END
    Take Screenshot

*** Tasks ***
Download Statement
    FOR    ${account}    IN    @{ACCOUNTS}
        ${account}
    END
